#!/usr/bin/env python


import sys

import gobject
import dbus
import dbus.service

import shiny
#import commandline



def print_prompt():
    print '> ',
    sys.stdout.flush()
def print_reply(reply):
    print '< ', reply
    print_prompt()
def print_event(event):
    print 
    print '#', event
    print_prompt()
def print_error(*args, **kwargs):
    print args, kwargs


def invoke(command, reply_cb=print_reply):
    """Sends command to skype, and prints the reply."""
    skype = init_skype()
    skype.Invoke(command, reply_handler=print_reply, error_handler=print_error)


class CallbackObject(dbus.service.Object):
    """This is for getting replies out of skype."""
    def __init__(self):
        dbus.service.Object.__init__(self, bus, '/com/Skype/Client')
    @dbus.service.method('com.Skype.API.Client', in_signature='v')
    def Notify(self, event):
        print_event(event)


@shiny.only_once
def init_skype(name='console'):
    """creates the global skypeapi object, to be used like
    skype.Invoke() and registers /com/Skype/Client Notify()."""
    global skype
    global callback_object
    bus = init_dbus()
    callback_object = CallbackObject()
    skype_object = bus.get_object('com.Skype.API', '/com/Skype')
    skype = dbus.Interface(skype_object, 'com.Skype.API')
    assert skype.Invoke('NAME console') == 'OK'
    assert skype.Invoke('PROTOCOL 5') == 'PROTOCOL 5'
    return skype

@shiny.only_once
def init_dbus():
    """Creates the global bus object, which is a dbus.SessionBus"""
    global bus
    from dbus.mainloop.glib import DBusGMainLoop
    DBusGMainLoop(set_as_default=True)
    bus = dbus.SessionBus()
    return bus





def apply_substitutions(line):
    """Hook for doing variable substitiution etc. at a later date."""
    return line


def handle_input(fd, _):
    """Called by dbus for every line of input. Just invokes commands and waits for replies."""
    line=fd.readline()
    if line=='': # no \n. probably EOF
        gobject.idle_add(exit)
        return False # Don't ask for more input
    else:
        command = apply_substitutions(line)
        invoke(command)
        return True

def main():
    """This provides a console for skype commands, using the skype api."""
    init_skype()
    gobject.io_add_watch(sys.stdin, gobject.IO_IN, handle_input)
    print_prompt()
    loop = gobject.MainLoop()
    loop.run()


if __name__ == "__main__":
    #commandline.run_as_main(main)
    main()
